@@ -13,7 +13,7 @@ def test_create_file(tmp_path):
     custom_labels_dict = {'hello': 'world', 'good': {'bye': 'bro'}}
     custom_labels.write_text(json.dumps(custom_labels_dict, indent=4))
 
-    str_label = 'my_str_label'
+    str_label = 'my:str?label*'
     user = 'my_user'
     basename = 'my_basename.json'
 
@@ -32,7 +32,7 @@ def test_create_file(tmp_path):
     assert d[CaperLabels.KEY_CAPER_BACKEND] == backend
     assert d['hello'] == 'world'
     assert d['good']['bye'] == 'bro'
-    assert d[CaperLabels.KEY_CAPER_STR_LABEL] == str_label
+    assert d[CaperLabels.KEY_CAPER_STR_LABEL] == 'my_str_label_'
     assert d[CaperLabels.KEY_CAPER_USER] == user
     assert os.path.basename(f) == basename
     assert os.path.dirname(f) == str(tmp_path)
